#ifndef WLR_VERSION_H
#define WLR_VERSION_H

#define WLR_VERSION_STR "0.12.0"

#define WLR_VERSION_MAJOR 0
#define WLR_VERSION_MINOR 12
#define WLR_VERSION_MICRO 0

#define WLR_VERSION_NUM ((WLR_VERSION_MAJOR << 16) | (WLR_VERSION_MINOR << 8) | WLR_VERSION_MICRO)

/* #undef WLR_VERSION_API_CURRENT */
/* #undef WLR_VERSION_API_REVISION */
/* #undef WLR_VERSION_API_AGE */

#endif
